##################################
# Description: creates records according to Grid Monitoring Probe Specification v0.91
# Author: Piotr Nyczyk(base code), Daniel Rodrigues
# Institute: CERN
# Created: January 08
# License:
# $Id: record.py 459 2008-09-07 13:47:00Z /DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=jamesc/CN=380618/CN=James Casey $
##################################

from __future__ import generators
import sys
import exceptions
import logging

logger=logging.getLogger('msg.record')

def load_records(inFiles):
    logger.info('Loading from : %s'%inFiles)
    if '-' in inFiles:
        inFiles.remove('-')
        inStreams = map(open, inFiles)
        inStreams.append(sys.stdin)
    else:
        inStreams = map(open, inFiles)
    return _chain(*map(_load_all, inStreams))


def next_record(input):
    """Parse a record out of an input stream.  Returns empty if there are no more records"""
    record=Record()
    key=''
    value=''
    multiline=False
    line=True
    while line:
        line=input.readline()
        line = line.strip()
        logger.debug("Have Line: '%s'"%line)
        if line[:1]=='#':
            continue
        if not line:
            continue
        if line=='EOT':
            break
        elif ':' in line and not multiline:
            key,value=map(str.strip, line.split(':',1))
            key=key.lower()
            if key=="detailsdata":
                multiline=True
            if value=='EOT':
                multiline=True
                value=''
            else:
                if ( len( value ) > 0 and ( value[0].isalnum() or value[0] == '/' ) ) :
                    record[key]=value
                else :
                    raise RecordSyntaxException( 'Syntax error: Record \'%s\' for key \'%s\' not according to specification.' % (value , key) )
        elif multiline:
            if len(value) > 0 and value[-1]!='\n':
                value+='\n'
            value+=line
        else:
            logger.error('Syntax error while parsing record: %s'%line)
    if multiline:
        record[key]=value
    return record
        
class Record(dict):
    """A record is simply a dict, which has some extra methods for formatting"""
    def dumps(self):
        logger = logging.getLogger('simplePublish.record')
        s=''
        multiline=None
        for k,v in self.items():
            if k.lower() == 'detailsdata' :
                multiline=k,v
            elif '\n' in v:
                if multiline:
                    logger.warning('Found additional multiline values - Ignoring...')
                else:
                    multiline=k,v
            else:
                s+=k+': '+v+'\n'
        if multiline:
            k,v=multiline
            if v[-1]!='\n':
                v+='\n'
            s+=k+':'+v
        s+='EOT\n'
        return s


class RecordSyntaxException(exceptions.Exception) :
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)



def _load_all(input):
    """A generator for an input stream, which produces Record objects"""
    record=True
    while record:
        record=next_record(input)
        if record != {}:
            yield record
            
            
def _chain(*iterables):
    """A generator over a set of iterators (here it's Record objects)"""
    for it in iterables:
        for element in it:
            yield element
